@@ -400,7 +400,7 @@ print_idle_cb (YelpPrintInfo *info)
 					      GTK_MESSAGE_WARNING,
 					      GTK_BUTTONS_OK,
 					      _("An error "
-						"occured while printing")
+						"occurred while printing")
 					      );
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 					      _("It was not possible to "
@@ -598,8 +598,10 @@ yelp_print_preview (YelpPrintInfo *info)
      * have to actually show it now :(
      */
     if (!info->previewed) {
+	GtkWidget *box;
+
 	info->previewed = TRUE;
-	GtkWidget *box = gtk_toolbar_new ();
+	box = gtk_toolbar_new ();
 	
 	gtk_box_pack_start (GTK_BOX (info->content_box), box, FALSE, FALSE, 0);
 	info->GoFirst = GTK_WIDGET (gtk_tool_button_new_from_stock (GTK_STOCK_GOTO_FIRST));
