@@ -137,6 +137,7 @@ static gboolean      process_mandir_pending    (YelpTocPager      *pager);
 #ifdef ENABLE_INFO
 static gboolean      process_info_pending      (YelpTocPager      *pager);
 #endif
+static gboolean      process_cleanup           (YelpTocPager      *pager);
 
 static void          toc_add_doc_info          (YelpTocPager      *pager,
 						YelpDocInfo       *doc_info);
@@ -364,6 +365,7 @@ toc_process_pending (YelpTocPager *pager)
 	process_info_pending,
 	process_xslt,
 #endif
+	process_cleanup,
 	NULL
     };
     static gint process_i = 0;
@@ -375,6 +377,8 @@ toc_process_pending (YelpTocPager *pager)
 
     readd = priv->pending_func(pager);
 
+    /* if the pending function returned FALSE, then we move on to
+     * calling the next function */
     if (!readd)
 	priv->pending_func = process_funcs[++process_i];
 
@@ -585,11 +589,15 @@ add_man_page_to_toc (YelpTocPager *pager, gchar *dirname, gchar *filename)
 	    c1 = NULL;
     }
 
+    /* c1 points to the file extension, either .gz or .bz2 at this point 
+     * or is NULL if the filename does not have either extension */
     if (c1)
 	c2 = g_strrstr_len (filename, c1 - filename, ".");
     else
 	c2 = g_strrstr (filename, ".");
 
+    /* c2 points to the period preceding the man page section in the 
+     * filename at this point */
     if (c2) {
 	manname = g_strndup (filename, c2 - filename);
 	if (c1)
@@ -735,12 +743,17 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 	    dirname = xmlNodeListGetString (manindex_xml, 
 	                                    node->xmlChildrenNode, 1);
 	    
-	    if (g_stat ((gchar *)dirname, &buf) < 0)
+	    /* if we can't stat the dirname for some reason, then skip adding
+	     * this directory to the TOC */
+	    if (g_stat ((gchar *)dirname, &buf) < 0) {
 		g_warning ("Unable to stat dir: \"%s\"\n", dirname);
-	    
+	    	continue;
+	    }
+
 	    /* FIXME: need some error checking */
 	    mtime = (time_t) atoi ((gchar *)dirmtime);
-	    
+
+	    xmlFree (dirmtime);
 	    /* see if directory mtime has changed - if so recreate
 	     * the directory node */
 	    if (buf.st_mtime > mtime) {
@@ -769,12 +782,12 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 			add_man_page_to_toc (pager, (gchar *)dirname, filename);
 			priv->manpage_count++;
 		    }
+
+		    g_dir_close (dir);
 		}
 
 		/* we replace the node in the tree */
 		xmlReplaceNode (dirnode, newNode);
-
-		g_dir_close (dir);
 	    
 	    /* otherwise just read from the index file */
 	    } else {
@@ -790,8 +803,12 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 
 		    add_man_page_to_toc (pager, (gchar *)dirname, (gchar *)manpage);
 		    priv->manpage_count++;
+		    xmlFree (manpage);
 	        }
+	    xmlXPathFreeObject (objmanpages);
 	    }
+	    xmlFree (dirname);
+	    xmlXPathFreeObject (objdirmtime);
 	}
 
 	if (priv->man_manhash) {
@@ -804,6 +821,8 @@ create_toc_from_index (YelpTocPager *pager, gchar *index_file)
 	create_manindex_file (index_file, manindex_xml);
     }
 
+    xmlXPathFreeContext (xpath);
+    xmlXPathFreeObject (objsect);
     xmlFree (manindex_xml);
     
     return 1;
@@ -836,11 +855,17 @@ process_mandir_pending (YelpTocPager *pager)
 	for (i = 0; i < obj->nodesetval->nodeNr; i++) {
 	    xmlNodePtr node = obj->nodesetval->nodeTab[i];
 	    xmlChar *sect = xmlGetProp (node, BAD_CAST "sect");
-	    if (sect)
-		g_hash_table_insert (priv->man_secthash, sect, node);
+
+	    if (sect) {
+		gchar **sects = g_strsplit ((gchar *)sect, " ", 0);
+
+		for (j = 0; sects[j] != NULL; j++)
+		    g_hash_table_insert (priv->man_secthash, sects[j], node);
+	    }
 
 	    xml_trim_titles (node);
 	}
+	xmlXPathFreeContext (xpath);
     }
 
     if (!index_file)
@@ -1021,8 +1046,8 @@ process_info_pending (YelpTocPager *pager)
 	    continue;
 	} else {
 	    GIOChannel *channel;
-	    gchar *str;
-	    int len;
+	    gchar *str = NULL;
+	    gsize len;
 	    gchar ** files;
 	    gchar ** ptr;
 	    xmlNodePtr tmp;
@@ -1043,7 +1068,7 @@ process_info_pending (YelpTocPager *pager)
 	    }
 	    
 	    channel = yelp_io_channel_new_file (filename, NULL);
-	    g_io_channel_read_to_end (channel, &str, (gsize *) &len, NULL);
+	    g_io_channel_read_to_end (channel, &str, &len, NULL);
 	    g_io_channel_shutdown (channel, FALSE, NULL);
 	    
 	    files = g_strsplit (str, "\n", -1);
@@ -1056,10 +1081,10 @@ process_info_pending (YelpTocPager *pager)
 		} else {
 		    if (g_str_has_prefix (*ptr, "*")) {
 			/* A menu item */
-			gchar *tooltip;
-			gchar *fname;
-			gchar *desc;
-			gchar *part1, *part2, *part3;
+			gchar *tooltip = NULL;
+			gchar *fname = NULL;
+			gchar *desc = NULL;
+			gchar *part1 = NULL, *part2 = NULL, *part3 = NULL;
 			gchar * path = *ptr;
 			gchar **nextline = ptr;
 			nextline++;
@@ -1067,14 +1092,22 @@ process_info_pending (YelpTocPager *pager)
 			 * split everything up...
 			 */
 			path++; 
+			if (!path)
+			    goto done;
 			part1 = strchr (path, ':');
+			if (!part1)
+			    goto done;
 			part2 = g_strndup (path, part1-path);
 			tooltip = g_strdup (part2);
 			tooltip = g_strstrip (tooltip);
-			tooltip = g_strconcat (_("Read info page for "), tooltip, NULL);
+			tooltip = g_strdup_printf (_("Read info page for %s"), tooltip);
 			path = part1+1;
 			part1 = strchr (path, ')');
+			if (!part1)
+			    goto done;
 			part1 = strchr (part1, '.');
+			if (!part1)
+			    goto done;
 			part3 = g_strndup (path, part1-path);
 			part1++;
 			desc = g_strdup (part1);
@@ -1108,6 +1141,7 @@ process_info_pending (YelpTocPager *pager)
 			    xmlNewChild (tmp, NULL, BAD_CAST "description",
 					 BAD_CAST desc);
 			}
+		    done:
 			g_free (part2);
 			g_free (part3);
 			g_free (tooltip);
@@ -1177,28 +1211,31 @@ process_read_menu (YelpTocPager *pager)
     for (i = 0; i < obj->nodesetval->nodeNr; i++) {
 	xmlNodePtr node = obj->nodesetval->nodeTab[i];
 	xmlChar *icon = NULL;
-
-#ifdef ENABLE_MAN
+#ifdef ENABLE_MAN_OR_INFO
 	xmlChar *id = NULL;
+	xmlNodePtr cmdhelp;
+	xmlNodePtr newnode;
 	id = xmlGetProp (node, BAD_CAST "id");
 	if (!xmlStrcmp (id, BAD_CAST "index")) {
-	    xmlNodePtr new = xmlNewChild (node, NULL, BAD_CAST "toc", NULL);
-	    xmlNewNsProp (new, NULL, BAD_CAST "id", BAD_CAST "Man");
-	    xmlNewChild (new, NULL, BAD_CAST "title", 
-			 BAD_CAST _("Manual Pages"));
-	}
-#endif
-
+	    cmdhelp = xmlNewChild (node, NULL, BAD_CAST "toc", NULL);
+	    xmlNewNsProp (cmdhelp, NULL, BAD_CAST "id", BAD_CAST "ManInfoHolder");
+	    xmlNewChild (cmdhelp, NULL, BAD_CAST "title", 
+			 BAD_CAST _("Command Line Help"));
+#ifdef ENABLE_MAN
+	newnode = xmlNewChild (cmdhelp, NULL, BAD_CAST "toc", NULL);
+	xmlNewNsProp (newnode, NULL, BAD_CAST "id", BAD_CAST "Man");
+	xmlNewChild (newnode, NULL, BAD_CAST "title", 
+		     BAD_CAST _("Manual Pages"));
+#endif // ENABLE_MAN
 #ifdef ENABLE_INFO
-	xmlChar *infoid = NULL;
-	infoid = xmlGetProp (node, BAD_CAST "id");
-	if (!xmlStrcmp (infoid, BAD_CAST "index")) {
-	    xmlNodePtr new = xmlNewChild (node, NULL, BAD_CAST "toc", NULL);
-	    xmlNewNsProp (new, NULL, BAD_CAST "id", BAD_CAST "Info");
-	    xmlNewChild (new, NULL, BAD_CAST "title", 
-			 BAD_CAST _("GNU Info Pages"));
+	newnode = xmlNewChild (cmdhelp, NULL, BAD_CAST "toc", NULL);
+	xmlNewNsProp (newnode, NULL, BAD_CAST "id", BAD_CAST "Info");
+	xmlNewChild (newnode, NULL, BAD_CAST "title", 
+		     BAD_CAST _("GNU Info Pages"));
+#endif //ENABLE_INFO
 	}
-#endif
+	xmlFree (id);
+#endif //ENABLE_MAN_OR_INFO
 
 	xml_trim_titles (node);
 
@@ -1289,7 +1326,12 @@ process_xslt (YelpTocPager *pager)
 
     if (!priv->toc_doc)
 	return FALSE;
-    priv->stylesheet = xsltParseStylesheetFile (BAD_CAST TOC_STYLESHEET);
+
+    /* only create and parse the stylesheet on the first call to this function */
+    if (!priv->stylesheet) {
+	priv->stylesheet = xsltParseStylesheetFile (BAD_CAST TOC_STYLESHEET);
+    }
+
     if (!priv->stylesheet) {
 	g_set_error (&error, YELP_ERROR, YELP_ERROR_PROC,
 		     _("The table of contents could not be processed. The "
@@ -1348,10 +1390,6 @@ process_xslt (YelpTocPager *pager)
 	xmlFreeDoc (priv->toc_doc);
 	priv->toc_doc = NULL;
     }
-    if (priv->stylesheet) {
-	xsltFreeStylesheet (priv->stylesheet);
-	priv->stylesheet = NULL;
-    }
     if (priv->transformContext) {
 	xsltFreeTransformContext (priv->transformContext);
 	priv->transformContext = NULL;
@@ -1360,6 +1398,21 @@ process_xslt (YelpTocPager *pager)
     return FALSE;
 }
 
+static gboolean
+process_cleanup (YelpTocPager *pager)
+{
+    YelpTocPagerPriv *priv = pager->priv;
+
+    /* cleanup the stylesheet used to process the toc */
+    if (priv->stylesheet) {
+	xsltFreeStylesheet (priv->stylesheet);
+	priv->stylesheet = NULL;
+    }
+
+    /* we only ever want to run this function once, so always return false */
+    return FALSE;
+}
+
 static void
 toc_add_doc_info (YelpTocPager *pager, YelpDocInfo *doc_info)
 {
