@@ -509,11 +509,11 @@ window_init (YelpWindow *window)
 static void
 window_dispose (GObject *object)
 {
-	YelpWindow *window = YELP_WINDOW (object);
+    YelpWindow *window = YELP_WINDOW (object);
 
-	window_disconnect (YELP_WINDOW (window));
+    window_disconnect (YELP_WINDOW (window));
 
-	parent_class->dispose (object);
+    parent_class->dispose (object);
 }
 
 static void
@@ -1155,6 +1155,9 @@ window_populate (YelpWindow *window)
     gtk_widget_show (priv->html_pane);
     gtk_widget_show (priv->pane);
     gtk_widget_show (priv->main_box);
+    gtk_container_set_focus_child (GTK_CONTAINER (window),
+				   GTK_WIDGET (priv->html_view));
+
 }
 
 static void
@@ -1671,17 +1674,20 @@ window_disconnect (YelpWindow *window)
     g_return_if_fail (YELP_IS_WINDOW (window));
 
     priv = window->priv;
-    pager = yelp_doc_info_get_pager (priv->current_doc);
+    
+    if (priv && priv->current_doc) {
+	pager = yelp_doc_info_get_pager (priv->current_doc);
+    }
 
     if (GTK_WIDGET (window)->window)
 	gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
 
-    if (window->priv->toc_pause > 0) {
-	window->priv->toc_pause--;
+    if (priv && priv->toc_pause > 0) {
+	priv->toc_pause--;
 	yelp_toc_pager_unpause (yelp_toc_pager_get ());
     }
 
-    if (window->priv->current_doc) {
+    if (priv && priv->current_doc) {
 	if (priv->start_handler) {
 	    g_signal_handler_disconnect (pager,
 					 priv->start_handler);
@@ -1708,7 +1714,7 @@ window_disconnect (YelpWindow *window)
 	    priv->finish_handler = 0;
 	}
     }
-    if (priv->idle_write) {
+    if (priv && priv->idle_write) {
 	gtk_idle_remove (priv->idle_write);
 	priv->idle_write = 0;
     }
@@ -1778,10 +1784,19 @@ pager_page_cb (YelpPager *pager,
     if (yelp_pager_page_contains_frag (pager,
 				       page_id,
 				       window->priv->current_frag)) {
-	window_disconnect (window);
-
 	page = (YelpPage *) yelp_pager_get_page (pager, page_id);
-	window_handle_page (window, page);
+
+	/* now that yelp automatically inserts the id="index" attribute
+	 * on the root element of a document, the _contains_frag function
+	 * is no longer a good indication of whether a section exists.
+	 * Therefore if the returned page is NULL, then the stylesheets
+	 * were not able to create a "page" from this document through the
+	 * exsl:document extension (see yelp_xslt_document())
+	 * -Brent Smith, 1/4/2006 */
+	if (page) {
+	    window_disconnect (window);
+	    window_handle_page (window, page);
+	}
     }
 }
 
